import java.awt.Polygon;

/**
 * Zeichnet ein Parallelogramm auf die Leinwand.
 * 
 * @author Bernd Gramlich nach einer Vorlage von Michael Klling und David J. Barnes 
 * @version 2008.05.14
 */
public class Parallelogramm
{
    /*
     * Die x- und y-Koordinaten der vier Eckpunkte werden
     * in jeweils einem Feld gespeichert.
     */
    private int[] x;
    private int[] y;
    
    /*
     * Farbe und Sichtbarkeit.
     */
    private String farbe;
    private boolean sichtbar;

    /**
     * Konstruktor der Klasse Parallelogramm.
     * Erzeugt ein neues Parallelogramm in Standardgre und Standardfarbe.
     */
    public Parallelogramm ()
    {
        /*
         * Erzeuge die Felder fr die x- und die y-Koordinaten der Eckpunkte. 
         */
        x = new int[4];
        y = new int[4];
        
        /*
         * Whle Standardwerte fr die x-Koordinaten der ersten drei Eckpunkte.
         */
        x[0] = 0;
        y[0] = 30;
        x[1] = 40;
        y[1] = 60;
        x[2] = 80;
        y[2] = 50;
        
        /*
         * Frbe das Parallelogramm grau und lasse es zunchst unsichtbar.
         */
        farbe         = "grau";
        sichtbar      = false;
    }
    
	/**
	 * Mache dieses Parallelogramm sichtbar.
	 */
	public void macheSichtbar ()
	{
		sichtbar = true;
		zeichne ();
	}

	/**
	 * Mache dieses Parallelogramm unsichtbar.
	 */
	public void macheUnsichtbar ()
	{
		loesche ();
		sichtbar = false;
	}
	
	/**
	 * Setze die Eckpunkte des Parallelogramms.
	 * Dazu werden die x- und die y-Koordinaten der ersten drei
	 * Eckpunkte bentigt. Die Koordinaten des vierten Eckpunkts
	 * werden vor dem Zeichnen automatisch berechnet.
	 */
	public void setzeEckpunkte (int x1, int y1, int x2, int y2, int x3, int y3)
	{
	    /*
	     * bernimm die gewnschten Koordinaten in die Koordinatenfelder.
	     */
		x[0] = x1;
		y[0] = y1;
		x[1] = x2;
		y[1] = y2;
		x[2] = x3;
		y[2] = y3;
		
		/*
		 * Zeichne das Parallelogramm neu.
		 */
		zeichne ();
	}

	/**
	 * ndere die Farbe dieses Parallelogramms in 'neueFarbe'. Gltige Angaben
	 * sind "rot", "gelb", "blau", "grn", "lila", "schwarz", "wei", "grau"
	 * "orange", "pink" und "trkis".
	 */
	public void setzeFarbe (String neueFarbe)
	{
		farbe = neueFarbe;
		zeichne ();
	}
	
	/**
	 * Verschiebe dieses Parallelogramm waagrecht um 'entfernung' Bildschirmpunkte.
	 */
	public void verschiebeWaagrecht (int entfernung)
	{
		loesche ();
		x[0] += entfernung;		
		x[1] += entfernung;
		x[2] += entfernung;
		zeichne ();
	}

	/**
	 * Verschiebe dieses Parallelogramm senkrecht um 'entfernung' Bildschirmpunkte.
	 */
	public void verschiebeSenkrecht (int entfernung)
	{
		loesche ();
		y[0] += entfernung;		
		y[1] += entfernung;
		y[2] += entfernung;
		zeichne ();
	}

	/**
	 * Gleite dieses Parallelogramm waagrecht um 'entfernung' Bildschirmpunkte.
	 */
	public void gleiteWaagrecht (int entfernung)
	{
		int delta;

		if (entfernung < 0)
		{
			delta = -1;
			entfernung = -entfernung;
		}
		else
		{
			delta = 1;
		}

		for (int i = 0; i < entfernung; i++)
		{
			x[0] += delta;
			x[1] += delta;
			x[2] += delta;
			zeichne ();
		}
	}

	/**
	 * Gleite dieses Parallelogramm senkrecht um 'entfernung' Bildschirmpunkte.
	 */
	public void gleiteSenkrecht (int entfernung)
	{
		int delta;

		if (entfernung < 0)
		{
			delta = -1;
			entfernung = -entfernung;
		}
		else
		{
			delta = 1;
		}

		for (int i = 0; i < entfernung; i++)
		{
			y[0] += delta;
			y[1] += delta;
			y[2] += delta;
			zeichne ();
		}
	}
	
	/**
	 * Zeichne dieses Parallelogramm auf die Leinwand.
	 */
	private void zeichne ()
	{
		if (sichtbar)
		{
		    /*
		     * Berechne die Koordinaten des vierten Eckpunkts.
		     */
		    x[3] = x[2] + x[0] - x[1];
		    y[3] = y[2] + y[0] - y[1];
		    
		    /*
		     * Zeichne nun das Parallelogramm.
		     */
			Leinwand leinwand = Leinwand.gibLeinwand ();
			leinwand.zeichne (this, farbe, new Polygon (x, y, 4));
			leinwand.warte (10);
		}
	}

	/**
	 * Lsche dieses Parallelogramm von der Leinwand.
	 */
	private void loesche ()
	{
		if (sichtbar)
		{
			Leinwand leinwand = Leinwand.gibLeinwand ();
			leinwand.entferne (this);
		}
	}
	
	/**
	 * Nenne den Flcheninhalt des Parallelogramms.
	 */
	public double gibFlaecheninhalt ()
	{
	    return Math.abs ((x[0] - x[1]) * (y[2] - y[1]) - (y[0] - y[1]) * (x[2] - x[1]));
	}
		
	/**
	 * Nenne die Seitenlnge zwischen dem ersten und dem zweiten Eckpunkt.
	 */
	public double gibErsteSeitenlaenge ()
	{
	    return Math.sqrt ((x[0] - x[1]) * (x[0] - x[1]) + (y[0] - y[1]) * (y[0] - y[1]));
	}
			
	/**
	 * Nenne die Seitenlnge zwischen dem zweiten und dem dritten Eckpunkt.
	 */
	public double gibZweiteSeitenlaenge ()
	{
	    return Math.sqrt ((x[2] - x[1]) * (x[2] - x[1]) + (y[2] - y[1]) * (y[2] - y[1]));
	}
}